/*
 * File:	WIN_listctrl.h
 * Purpose:	List control
 * Author:	Julian Smart
 * Changed:     Robert Roebling (generic implementation for wxXt)
 * Created:	1997
 * Copyright:   L-GPL
 */

#ifndef __LISTCTRLH__
#define __LISTCTRLH__

#ifdef __GNUG__
#pragma interface
#endif

#include "ImagList.h"

//-----------------------------------------------------------------------------
// classes
//-----------------------------------------------------------------------------

class wxListItem;
class wxListEvent;
class wxListCtrl;

//-----------------------------------------------------------------------------
// types
//-----------------------------------------------------------------------------

// type of compare function for wxListCtrl sort operation
typedef int (*wxListCtrlCompare)(const long item1, const long item2, long sortData);

//-----------------------------------------------------------------------------
// wxListCtrl flags
//-----------------------------------------------------------------------------

#define wxLC_ICON            0x0004
#define wxLC_SMALL_ICON      0x0008
#define wxLC_LIST            0x0010
#define wxLC_REPORT          0x0020
#define wxLC_ALIGN_TOP       0x0040
#define wxLC_ALIGN_LEFT      0x0080
#define wxLC_AUTOARRANGE     0x0100  // not supported in wxXt
#define wxLC_USER_TEXT       0x0200  // not supported in wxXt (how does it work?)
#define wxLC_EDIT_LABELS     0x0400
#define wxLC_NO_HEADER       0x0800  // not supported in wxXt
#define wxLC_NO_SORT_HEADER  0x1000  // not supported in wxXt
#define wxLC_SINGLE_SEL      0x2000
#define wxLC_SORT_ASCENDING  0x4000  
#define wxLC_SORT_DESCENDING 0x8000  // not supported in wxXt

#define wxLC_MASK_TYPE       (wxLC_ICON | wxLC_SMALL_ICON | wxLC_LIST | wxLC_REPORT)
#define wxLC_MASK_ALIGN      (wxLC_ALIGN_TOP | wxLC_ALIGN_LEFT)
#define wxLC_MASK_SORT       (wxLC_SORT_ASCENDING | wxLC_SORT_DESCENDING)

// Omitted because (a) too much detail (b) not enough style flags
// #define wxLC_NO_SCROLL
// #define wxLC_NO_LABEL_WRAP
// #define wxLC_OWNERDRAW_FIXED
// #define wxLC_SHOW_SEL_ALWAYS

// Mask flags to tell app/GUI what fields of wxListItem are valid
#define wxLIST_MASK_STATE           0x0001
#define wxLIST_MASK_TEXT            0x0002
#define wxLIST_MASK_IMAGE           0x0004
#define wxLIST_MASK_DATA            0x0008
#define wxLIST_SET_ITEM             0x0010
#define wxLIST_MASK_WIDTH           0x0020
#define wxLIST_MASK_FORMAT          0x0040

// State flags for indicating the state of an item
#define wxLIST_STATE_DONTCARE       0x0000
#define wxLIST_STATE_DROPHILITED    0x0001  // not supported in wxXt
#define wxLIST_STATE_FOCUSED        0x0002
#define wxLIST_STATE_SELECTED       0x0004  
#define wxLIST_STATE_CUT            0x0008  // not supported in wxXt

// Hit test flags, used in HitTest // wxXt suppots 20 and 80
#define wxLIST_HITTEST_ABOVE            0x0001  // Above the client area.
#define wxLIST_HITTEST_BELOW            0x0002  // Below the client area.
#define wxLIST_HITTEST_NOWHERE          0x0004  // In the client area but below the last item.
#define wxLIST_HITTEST_ONITEMICON       0x0020  // On the bitmap associated with an item.
#define wxLIST_HITTEST_ONITEMLABEL      0x0080  // On the label (string) associated with an item.
#define wxLIST_HITTEST_ONITEMRIGHT      0x0100  // In the area to the right of an item.
#define wxLIST_HITTEST_ONITEMSTATEICON  0x0200  // On the state icon for a tree view item that is in a user-defined state.
#define wxLIST_HITTEST_TOLEFT           0x0400  // To the right of the client area.
#define wxLIST_HITTEST_TORIGHT          0x0800  // To the left of the client area.

#define wxLIST_HITTEST_ONITEM (wxLIST_HITTEST_ONITEMICON | wxLIST_HITTEST_ONITEMLABEL | wxLIST_HITTEST_ONITEMSTATEICON)



// Flags for GetNextItem  // always wxLIST_NEXT_ALL in wxXt
enum {
    wxLIST_NEXT_ABOVE,          // Searches for an item above the specified item
    wxLIST_NEXT_ALL,            // Searches for subsequent item by index
    wxLIST_NEXT_BELOW,          // Searches for an item below the specified item
    wxLIST_NEXT_LEFT,           // Searches for an item to the left of the specified item
    wxLIST_NEXT_RIGHT,          // Searches for an item to the right of the specified item
};

// Alignment flags for Arrange  // always wxLIST_ALIGN_LEFT in wxXt
enum {
    wxLIST_ALIGN_DEFAULT,
    wxLIST_ALIGN_LEFT,
    wxLIST_ALIGN_TOP,
    wxLIST_ALIGN_SNAP_TO_GRID
};

// Column format  // always wxLIST_FORMAT_LEFT in wxXt
enum {
    wxLIST_FORMAT_LEFT,
    wxLIST_FORMAT_RIGHT,
    wxLIST_FORMAT_CENTRE,
    wxLIST_FORMAT_CENTER = wxLIST_FORMAT_CENTRE
};

// Autosize values for SetColumnWidth
enum {
    wxLIST_AUTOSIZE = -1,          // always 80 in wxXt (what else?)
    wxLIST_AUTOSIZE_USEHEADER = -2
};

// Flag values for GetItemRect
enum {
    wxLIST_RECT_BOUNDS,
    wxLIST_RECT_ICON,
    wxLIST_RECT_LABEL
};

// Flag values for FindItem  // not supported by wxXt
enum {
    wxLIST_FIND_UP,
    wxLIST_FIND_DOWN,
    wxLIST_FIND_LEFT,
    wxLIST_FIND_RIGHT
};

// Flag values for Set/GetImageList
enum {
    wxIMAGE_LIST_NORMAL, // Normal icons
    wxIMAGE_LIST_SMALL,  // Small icons
    wxIMAGE_LIST_STATE   // State icons: unimplemented (see WIN32 documentation)
                         // not implemented in wxXt (see non-existing documentation..)
};

//-----------------------------------------------------------------------------
// wxListItem
//-----------------------------------------------------------------------------

class wxListItem: public wxObject
{
  DECLARE_DYNAMIC_CLASS(wxListItem)

  public:
    long            m_mask;     // Indicates what fields are valid
    long            m_itemId;   // The zero-based item position
    int             m_col;      // Zero-based column, if in report mode
    long            m_state;    // The state of the item
    long            m_stateMask; // Which flags of m_state are valid (uses same flags)
    wxString        m_text;     // The label/header text
    int             m_image;    // The zero-based index into an image list
    long            m_data;     // App-defined data
    wxColour       *m_colour;   // only wxXt, not supported by Windows ;->

    // For columns only
    int             m_format;   // left, right, centre
    int             m_width;    // width of column

    wxListItem(void);
};

//-----------------------------------------------------------------------------
// wxListEvent
//-----------------------------------------------------------------------------

enum {
 wxEVT_COMMAND_LIST_BEGIN_DRAG = EVENT_TYPES_FIRST + 400,  // not supported in wxXt
 wxEVT_COMMAND_LIST_BEGIN_RDRAG,                           // not supported in wxXt
 wxEVT_COMMAND_LIST_BEGIN_LABEL_EDIT,   // not supported in wxXt
 wxEVT_COMMAND_LIST_END_LABEL_EDIT,
 wxEVT_COMMAND_LIST_DELETE_ITEM,     
 wxEVT_COMMAND_LIST_DELETE_ALL_ITEMS,   // not supported in wxXt
 wxEVT_COMMAND_LIST_GET_INFO,           // not supported in wxXt
 wxEVT_COMMAND_LIST_SET_INFO,           // not supported in wxXt
 wxEVT_COMMAND_LIST_ITEM_SELECTED,
 wxEVT_COMMAND_LIST_ITEM_DESELECTED,
 wxEVT_COMMAND_LIST_ITEM_FOCUSSED,
 wxEVT_COMMAND_LIST_ITEM_UNFOCUSSED,
 wxEVT_COMMAND_LIST_KEY_DOWN,
 wxEVT_COMMAND_LIST_INSERT_ITEM,        // not supported in wxXt
 wxEVT_COMMAND_LIST_COL_CLICK           // not supported in wxXt (boring)
};

class wxListEvent: public wxCommandEvent 
{
  DECLARE_DYNAMIC_CLASS(wxListEvent);

 public:
  wxListEvent(WXTYPE commandType = 0, int id = 0);

  int           m_code;
  long          m_itemIndex;
  long          m_oldItemIndex;
  int           m_col;
  Bool          m_cancelled;
  wxPoint       m_pointDrag;

  wxListItem    m_item;
};

//-----------------------------------------------------------------------------
// wxListCtrl
//-----------------------------------------------------------------------------

#include "WIN_rawlistctrl.h" // must be here for wxXt

class wxListCtrl: public wxRawListCtrl  
{
  DECLARE_DYNAMIC_CLASS(wxListCtrl);
  
  public:
  wxListCtrl(void);
  wxListCtrl( wxWindow *parent, int x = -1, int x = -1, int w = -1, int h = -1,
    const long style = 0, const char *name = "wxListCtrl");
  ~wxListCtrl(void);
//  Create()       // not needed for wxXt
  void SetBackgroundColour( const wxColour& col );
  Bool GetColumn( const int col, wxListItem& item );
  Bool SetColumn( const int col, wxListItem& item );
  int GetColumnWidth( const int col );
  Bool SetColumnWidth( const int col, const int width);
  int GetCountPerPage(void); // not the same in wxXt as in Windows, I think
//  wxText& GetEditControl(void) const; // not supported in wxXt
  Bool GetItem( wxListItem& info );
  Bool SetItem( wxListItem& info ) ;
  long SetItem( const long index, const int col, const wxString& label, const int imageId = -1 );
  int  GetItemState( const long item, const long stateMask );
  Bool SetItemState( const long item, const long state, const long stateMask); 
  Bool SetItemImage( const long item, const int image, const int selImage); 
  wxString GetItemText( const long item );
  void SetItemText( const long item, const wxString& str );
  long GetItemData( const long item );
  Bool SetItemData( const long item, long data );
  Bool GetItemRect( const long item, wxRectangle& rect, const int code = wxLIST_RECT_BOUNDS ); // not supported in wxXt
  Bool GetItemPosition( const long item, wxPoint& pos ) const; // not supported in wxXt
  Bool SetItemPosition( const long item, const wxPoint& pos ); // not supported in wxXt
  int GetItemCount(void);
  int GetItemSpacing( Bool isSmall );
  int GetSelectedItemCount(void);
//  wxColour GetTextColour(void) const; // wxXt has colours for every Item (see wxListItem)
//  void SetTextColour(const wxColour& col);
  long GetTopItem(void);
  void SetSingleStyle( const long style, const Bool add = TRUE ) ;
//  void SetWindowStyleFlag(const long style); // see below for wxXt
  void SetListStyleFlag(const long style);
//  void RecreateWindow(void);  // not needed for wxXt
  void RealizeChanges( void );  // whereas this is much needed in wxXt
  long GetNextItem(const long item, int geometry = wxLIST_NEXT_ALL, int state = wxLIST_STATE_DONTCARE);
  wxImageList *GetImageList(const int which);
  void SetImageList(wxImageList *imageList, const int which) ;
  Bool Arrange( const int flag = wxLIST_ALIGN_DEFAULT ); // always wxLIST_ALIGN_LEFT in wxXt
  Bool DeleteItem( const long item );
  Bool DeleteAllItems(void) ;
  Bool DeleteColumn( const int col );
//  wxText& Edit(const long item) ;  // not supported in wxXt
  Bool EnsureVisible(const long item) ;
  long FindItem(const long start, const wxString& str, const Bool partial = FALSE); // not supported in wxXt
  long FindItem(const long start, const long data); // not supported in wxXt
  long FindItem(const long start, const wxPoint& pt, const int direction); // not supported in wxXt
  long HitTest(const wxPoint& point, int& flags);
  long InsertItem(wxListItem& info);
  long InsertItem(const long index, const wxString& label);
  long InsertItem(const long index, const int imageIndex);
  long InsertItem(const long index, const wxString& label, const int imageIndex);
  long InsertColumn(const long col, wxListItem& info);
  long InsertColumn(const long col, const wxString& heading, const int format = wxLIST_FORMAT_LEFT,
    const int width = -1);
  Bool ScrollList(const int dx, const int dy);
  Bool SortItems(wxListCtrlCompare fn, long data);
  Bool Update(const long item);
  virtual Bool OnListNotify(wxListEvent& WXUNUSED(event)) { return FALSE; }
  void Command(wxCommandEvent& event) { ProcessCommand(event); };

protected:
//  wxText          m_textCtrl; // not supported in wxXt
  long              m_style;
  wxImageList *     m_imageListNormal; 
  wxImageList *     m_imageListSmall;  
  wxImageList *     m_imageListState;  // what's that ?

};

#endif // ListCtrlh
